## Function mate - Performs a mating between selected mother and
## father to produce offspring's allele set.
##
## dfr: a preprocessed PolyPatEx allele dataset
## moth: id of selected mother
## fath: id of selected father
## nPrefixCols: number of columns in dfr preceeding the allele data
##              (4 for dioecious data, 3 for monoecious)
## nLoci: number of loci in allele dataset
## ploidy: ploidy of dataset
##
mate <- function(moth,fath,nPrefixCols,nLoci,ploidy,dfr)
  {
    stopifnot(dim(dfr)[2]==nPrefixCols+nLoci*ploidy)
    progAlleleSet <- rep(NA,length=nLoci*ploidy)
    for (ll in 0:(nLoci-1)) {
      mothAlleles <- as.character(dfr[moth,(nPrefixCols + ll*ploidy) + 1:ploidy])
      fathAlleles <- as.character(dfr[fath,(nPrefixCols + ll*ploidy) + 1:ploidy])
      progAlleleSet[ll*ploidy + 1:ploidy] <- c(sample(mothAlleles,ploidy/2),
                                               sample(fathAlleles,ploidy/2))
    }
    stopifnot(all(!is.na(progAlleleSet)))
    return(progAlleleSet)
  }


## Simulate a (currently dioecious) dataset
##
## nLoci: number of loci in allele dataset
## ploidy: ploidy of dataset
## nAlleles: number of alleles per locus (constant across loci)
## nCandidates: number of fathers
## nMothers: number of mothers
## nProgenyPerMother: number of progeny per mother (constant across
##                    mothers)
## simName: (used as 'popn' entry)
simulateAlleleDataset <- function(ploidy,nLoci,nAlleles,nCandidates,
                                  nMothers,nProgenyPerMother,
                                  simName = "Simulation")
{
  ##
  stopifnot(is.numeric(ploidy))
  stopifnot(is.numeric(nLoci))
  stopifnot(is.numeric(nAlleles))
  stopifnot(is.numeric(nCandidates))
  stopifnot(is.numeric(nMothers))
  stopifnot(is.numeric(nProgenyPerMother))
  stopifnot(is.character(simName))
  ##
  ## Total number of progeny
  nProgeny <- nMothers*nProgenyPerMother
  ##
  ## Set up output dataset
  ##
  id <- c(paste("C",sprintf("%02d", 1:nCandidates),sep="_"),
          paste("M",sprintf("%02d", 1:nMothers),sep="_"))
  ##
  stopifnot(nMothers*nProgenyPerMother < 1000)
  if (nMothers*nProgenyPerMother >= 100)
    {
      id <- c(id,paste("P",
                       sprintf("%03d", 1:(nMothers*nProgenyPerMother)),
                       sep="_"))
    } else {
      id <- c(id,paste("P",
                       sprintf("%0d", 1:(nMothers*nProgenyPerMother)),
                       sep="_"))
    }
  ##
  ## Set up allele dataset template
  dfr <- data.frame(id)
  rownames(dfr) <- id
  rm(id)
  dfr$popn <- simName
  dfr$mother <- NA
  dfr$gender <- NA
  ## Number of prefix (i.e.. non-allele data) columns in the data set.
  nPrefixCols <- 3 + sum("gender" %in% names(dfr))
  ##
  ## Some convenient index/label sets
  candidates <- grep("^C",dfr$id)
  candidateLabels <- grep("^C",dfr$id,value=TRUE)
  mothers <- grep("^M",dfr$id)
  progenyLabels <- grep("^P",dfr$id,value=TRUE)
  mothers <- grep("^M",dfr$id)
  progenyLabels <- grep("^P",dfr$id,value=TRUE)
  alleleCols <- (nPrefixCols+1):(dim(dfr)[2])
  ##
  ## Total number of rows in the dataset
  nRows <- nCandidates + nMothers + nProgeny
  stopifnot(nRows == dim(dfr)[1])  ## Unnecessarily paranoid?
  ##
  ## Append columns for allele data to dataset template
  alleleNames <- paste(paste0("L",rep(1:nLoci,each=ploidy)),
                       paste0("A",rep(1:ploidy,times=nLoci)),
                       sep="_")
  mm <- matrix(NA,nrow = nRows,ncol=length(alleleNames))
  colnames(mm) <- alleleNames
  dfr <- cbind(dfr,as.data.frame(mm))
  rm(mm)
  ##
  ## Set up mother column
  dfr$mother <- c(rep(NA,nCandidates+nMothers),
                  rep(as.character(dfr$id[mothers]),each=nProgenyPerMother))
  ##
  ## Simulate candidate fathers
  dfr$gender[candidates] <- "M"  ## for Male
  for (cc in candidates) {
    for (ll in 0:(nLoci-1)) {
      dfr[cc,(nPrefixCols + ll*ploidy) + 1:ploidy] <- sample(1:nAlleles,
                                                             size=ploidy,
                                                             replace=TRUE,
                                                             prob=(1:nAlleles)/(nAlleles*(nAlleles+1)/2))
    }
  }
  ##
  ## Simulate mothers
  dfr$gender[mothers] <- "F"  ## for Male
  for (cc in mothers) {
    for (ll in 0:(nLoci-1)) {
      dfr[cc,(nPrefixCols + ll*ploidy) + 1:ploidy] <- sample(1:nAlleles,
                                                             size=ploidy,
                                                             replace=TRUE,
                                                             prob=(1:nAlleles)/(nAlleles*(nAlleles+1)/2))
    }
  }
  ##
  ## Create a list to store the fathers of the progeny
  progenyFathers <- vector("list", length(progenyLabels))
  names(progenyFathers) <- progenyLabels
  ##
  ## OK - now, for each progeny
  ##
  prog <- progenyLabels[1]
  moth <- dfr[prog,"mother"]
  fath <- sample(candidateLabels,1)
  progenyFathers[[prog]] <- fath
  head(progenyFathers)
  ##
  ##
  for (prog in progenyLabels)
    {
      moth <- dfr[prog,"mother"]
      fath <- sample(candidateLabels,1)
      progenyFathers[[prog]] <- fath
      head(progenyFathers)
      dfr[prog,-(1:nPrefixCols)] <- mate(moth,fath,nPrefixCols,nLoci,ploidy,dfr)
    }
  dfr$id <- as.character(dfr$id)
  return(list(aData=dfr,progenyFathers=progenyFathers))
}



